@@ -1,8 +1,15 @@
+"""
+main pyxtal module to create the pyxtal class
+"""
+
 # Standard Libraries
 from copy import deepcopy
 from random import choice, sample
-import numpy as np
 import itertools
+import numpy as np
+
+from ase import Atoms
+from pymatgen.core.structure import Structure, Molecule
 
 # PyXtal imports #avoid *
 from pyxtal.version import __version__
@@ -17,14 +24,17 @@ from pyxtal.crystal import (
     random_crystal_1D,
     random_crystal_2D,
 )
-from pyxtal.symmetry import Group, Wyckoff_position
+from pyxtal.symmetry import Group, Wyckoff_position, search_matched_position
+from pyxtal.operations import apply_ops, SymmOp, get_inverse
 from pyxtal.wyckoff_site import atom_site, mol_site, WP_merge
 from pyxtal.wyckoff_split import wyckoff_split
+from pyxtal.molecule import pyxtal_molecule
 from pyxtal.lattice import Lattice
-from pyxtal.operations import apply_ops
 from pyxtal.tolerance import Tol_matrix
 from pyxtal.io import read_cif, write_cif, structure_from_ext
 from pyxtal.XRD import XRD
+from pyxtal.constants import letters
+from pyxtal.viz import display_molecular, display_atomic
 
 # name = "pyxtal"
 
@@ -35,10 +45,10 @@ def print_logo():
 
     print(
         """
-             ______       _    _          _   
-            (_____ \     \ \  / /        | |   
-             _____) )   _ \ \/ / |_  ____| |  
-            |  ____/ | | | )  (|  _)/ _  | | 
+             ______       _    _          _
+            (_____ \     \ \  / /        | |
+             _____) )   _ \ \/ / |_  ____| |
+            |  ____/ | | | )  (|  _)/ _  | |
             | |    | |_| |/ /\ \ |_( (_| | |___
             |_|     \__  /_/  \_\___)__|_|_____)
                    (____/      """
@@ -51,12 +61,12 @@ def print_logo():
 
 class pyxtal:
     """
-    Class for handling atomic crystals based on symmetry constraints. 
+    Class for handling atomic crystals based on symmetry constraints
 
     Examples
     --------
 
-    To create a new structure instance, 
+    To create a new structure instance
 
     >>> from pyxtal import pyxtal
     >>> struc = pyxtal()
@@ -129,13 +139,26 @@ class pyxtal:
         self.diag = False
         self.numIons = None
         self.numMols = None
+        self.source = None
+        self.formula = None
+        self.species = None
+        self.group = None
+        self.lattice = None
+        self.dim = 3
+        self.factor = 1.0
+        self.PBC = [1,1,1]
+        if molecular:
+            self.molecules = []
+            self.mol_sites = []
+        else:
+            self.atom_sites = []
 
     def __str__(self):
         if self.valid:
-            s = "------Crystal from {:s}------".format(self.source)
+            s = "\n------Crystal from {:s}------".format(self.source)
             s += "\nDimension: {}".format(self.dim)
             s += "\nComposition: {}".format(self.formula)
-            if self.group.number in [7, 14, 15] and self.diag:
+            if self.group.number in [5, 7, 8, 9, 12, 13, 14, 15] and self.diag:
                 symbol = self.group.alias
             else:
                 symbol = self.group.symbol
@@ -220,26 +243,28 @@ class pyxtal:
             count += 1
             if self.molecular:
                 if dim == 3:
-                    struc = molecular_crystal(group, species, numIons, factor, 
+                    if numIons is None:
+                        numIons = [len(Group(group)[0])]*len(species)
+                    struc = molecular_crystal(group, species, numIons, factor,
                     lattice=lattice, sites=sites, conventional=conventional, diag=diag, tm=tm)
                 elif dim == 2:
-                    struc = molecular_crystal_2D(group, species, numIons, factor, 
+                    struc = molecular_crystal_2D(group, species, numIons, factor,
                     thickness=thickness, sites=sites, conventional=conventional, tm=tm)
                 elif dim == 1:
-                    struc = molecular_crystal_1D(group, species, numIons, factor, 
+                    struc = molecular_crystal_1D(group, species, numIons, factor,
                     area=area, sites=sites, conventional=conventional, tm=tm)
             else:
                 if dim == 3:
-                    struc = random_crystal(group, species, numIons, factor, 
+                    struc = random_crystal(group, species, numIons, factor,
                             lattice, sites, conventional, tm)
                 elif dim == 2:
-                    struc = random_crystal_2D(group, species, numIons, factor, 
+                    struc = random_crystal_2D(group, species, numIons, factor,
                             thickness, lattice, sites, conventional, tm)
                 elif dim == 1:
-                    struc = random_crystal_1D(group, species, numIons, factor, 
+                    struc = random_crystal_1D(group, species, numIons, factor,
                             area, lattice, sites, conventional, tm)
                 else:
-                    struc = random_cluster(group, species, numIons, factor, 
+                    struc = random_cluster(group, species, numIons, factor,
                             lattice, sites, tm)
             if force_pass:
                 quit = True
@@ -269,7 +294,6 @@ class pyxtal:
                 self.PBC = struc.PBC
                 self.source = 'random'
                 self.factor = struc.factor
-                self.number = struc.number
                 self._get_formula()
             except:
                 pass
@@ -280,18 +304,19 @@ class pyxtal:
         Load the seed structure from Pymatgen/ASE/POSCAR/CIFs
         Internally they will be handled by Pymatgen
         """
-        from ase import Atoms
-        from pymatgen import Structure
 
         if self.molecular:
-            from pyxtal.molecule import pyxtal_molecule
-            pmol = pyxtal_molecule(molecule).mol
+            pmol = pyxtal_molecule(molecule)#.mol
             struc = structure_from_ext(seed, pmol, relax_h=relax_h)
             if struc.match():
                 self.mol_sites = [struc.make_mol_site()]
+                #xyz = self.mol_sites[0].molecule.mol.cart_coords
+                #self.mol_sites[0].molecule = pyxtal_molecule(molecule)
+                #self.mol_sites[0].molecule.reset_positions(xyz)
                 self.group = Group(struc.wyc.number)
                 self.lattice = struc.lattice
-                self.molecules = [pyxtal_molecule(struc.molecule, symmetrize=False)]
+                #self.molecules = [pyxtal_molecule(struc.molecule, symmetrize=False)]
+                self.molecules = [self.mol_sites[0].molecule]
                 self.numMols = struc.numMols
                 self.diag = struc.diag
                 self.valid = True # Need to add a check function
@@ -350,26 +375,13 @@ class pyxtal:
             self.group = Group(number)
             matrix, ltype = sym_struc.lattice.matrix, self.group.lattice_type
             self.lattice = Lattice.from_matrix(matrix, ltype=ltype)
-            wp0 = self.group[0]
             atom_sites = []
             for i, site in enumerate(sym_struc.equivalent_sites):
-                pos = site[0].frac_coords 
+                pos = site[0].frac_coords
                 wp = Wyckoff_position.from_group_and_index(number, sym_struc.wyckoff_symbols[i])
                 specie = site[0].specie.number
-                match = False
-                for op in wp0:
-                    pos1 = op.operate(pos)
-                    pos0 = wp[0].operate(pos1)
-                    diff = pos1 - pos0
-                    diff -= np.round(diff)
-                    diff = np.abs(diff)
-                    #print(wp.letter, pos1, pos0, diff)
-                    if diff.sum()<1e-2:
-                        pos1 -= np.floor(pos1)
-                        match = True
-                        break
-                #print("============", match, wp.letter, pos, pos0)
-                if match:
+                pos1 = search_matched_position(self.group, wp, pos)
+                if pos1 is not None:
                     atom_sites.append(atom_site(wp, pos1, specie))
                 else:
                     break
@@ -440,7 +452,7 @@ class pyxtal:
             raise NotImplementedError("Does not support cluster for now")
         return N_pairs
 
-    def to_file(self, filename=None, fmt=None, permission='w', sym_num=None):
+    def to_file(self, filename=None, fmt=None, permission='w', sym_num=None, header="from_pyxtal"):
         """
         Creates a file with the given filename and file type to store the structure.
         By default, creates cif files for crystals and xyz files for clusters.
@@ -463,7 +475,7 @@ class pyxtal:
 
             if fmt == "cif":
                 if self.dim == 3:
-                    return write_cif(self, filename, "from_pyxtal", permission, sym_num=sym_num)
+                    return write_cif(self, filename, header, permission, sym_num=sym_num)
                 else:
                     pmg_struc = self.to_pymatgen()
                     if self.molecular:
@@ -528,7 +540,7 @@ class pyxtal:
                     if splitter.valid_split:
                         special = False
                         if self.molecular:
-                            for i, site in enumerate(self.mol_sites):
+                            for i in range(len(self.mol_sites)):
                                 for ops in splitter.H_orbits[i]:
                                     if len(ops) < len(splitter.H[0]):
                                         special = True
@@ -568,7 +580,7 @@ class pyxtal:
             return new_strucs
 
 
-    def subgroup_once(self, eps=0.1, H=None, permutations=None, group_type='t', max_cell=4):
+    def subgroup_once(self, eps=0.1, H=None, permutations=None, group_type='t', max_cell=4, mut_lat=True):
         """
         generate a structure with lower symmetry (for atomic crystals only)
 
@@ -600,7 +612,7 @@ class pyxtal:
                             return self._apply_substitution(splitter, permutations)
                         else:
                             #print("try to find the next subgroup")
-                            trail_struc = self._subgroup_by_splitter(splitter, eps=eps)
+                            trail_struc = self._subgroup_by_splitter(splitter, eps=eps, mut_lat=mut_lat)
                             multiple = sum(trail_struc.numIons)/sum(self.numIons)
                             max_cell = max([1, max_cell/multiple])
                             ans = trail_struc.subgroup_once(eps, H, permutations, group_type, max_cell)
@@ -612,16 +624,16 @@ class pyxtal:
                     if splitter.valid_split:
                         special = False
                         if self.molecular:
-                            for i, site in enumerate(self.mol_sites):
+                            for i in range(len(self.mol_sites)):
                                 for ops in splitter.H_orbits[i]:
                                     if len(ops) < len(splitter.H[0]):
                                         special = True
                                         break
                         if not special:
-                            return self._subgroup_by_splitter(splitter, eps=eps)
+                            return self._subgroup_by_splitter(splitter, eps=eps, mut_lat=mut_lat)
                     else:
                         #print("try to find the next subgroup")
-                        trail_struc = self._subgroup_by_splitter(splitter, eps=eps)
+                        trail_struc = self._subgroup_by_splitter(splitter, eps=eps, mut_lat=mut_lat)
                         multiple = sum(trail_struc.numIons)/sum(self.numIons)
                         max_cell = max([1, max_cell/multiple])
                         ans = trail_struc.subgroup_once(eps, H, None, group_type, max_cell)
@@ -630,7 +642,7 @@ class pyxtal:
             count += 1
         raise RuntimeError("Cannot find the splitter")
 
-    def _apply_substitution(self, splitter, permutations, single=False):
+    def _apply_substitution(self, splitter, permutations):
         """
         dummy function to apply the substitution
         """
@@ -639,7 +651,7 @@ class pyxtal:
         except:
             print(self)
             print(splitter)
-            print("---------------", len(splitter.H_orbits), len(splitter.G2_orbits), len(self.atom_sites))
+            print(len(splitter.H_orbits), len(splitter.G2_orbits), len(self.atom_sites))
             self._subgroup_by_splitter(splitter)
 
         site_ids = []
@@ -679,7 +691,7 @@ class pyxtal:
 
         Hs = dicts['subgroup']
         trans = dicts['transformation']
-        
+
         if idx is None:
             idx = []
             if not self.molecular:
@@ -693,7 +705,7 @@ class pyxtal:
                     # QZ: This loop needs a generalization!
                     if abs(np.linalg.det(tran[:3,:3])-1)>1e-3:
                         good = False
-                    elif self.group.number in [7, 14, 15] and self.diag and Hs[i]==4:
+                    elif self.group.number in [5, 7, 8, 9, 12, 13, 14, 15] and self.diag and Hs[i]==4:
                         good = False
                     elif self.group.number in [31] and Hs[i]==7:
                         good = False
@@ -703,8 +715,8 @@ class pyxtal:
             for id in idx:
                 if id >= len(Hs):
                     raise ValueError("The idx exceeds the number of possible splits")
-        
-        if H is not None: 
+
+        if H is not None:
             idx = [id for id in idx if Hs[id] == H]
 
         if len(idx) == 0:
@@ -719,17 +731,22 @@ class pyxtal:
 
         return idx, sites, t_types, k_types
 
-    def _subgroup_by_splitter(self, splitter, eps=0.05):
+    def _subgroup_by_splitter(self, splitter, eps=0.05, mut_lat=True):
         """
         transform the crystal to subgroup symmetry from a splitter object
+
+        Args:
+            splitter: wyckoff splitter object
+            eps (float): maximum atomic displacement in Angstrom
+            mut_lat (bool): whether or not mutate the lattice
         """
-        from pyxtal.molecule import Orientation
         lat1 = np.dot(splitter.R[:3,:3].T, self.lattice.matrix)
         multiples = np.linalg.det(splitter.R[:3,:3])
         new_struc = self.copy()
         new_struc.group = splitter.H
         lattice = Lattice.from_matrix(lat1, ltype=new_struc.group.lattice_type)
-        lattice=lattice.mutate(degree=eps, frozen=True)
+        if mut_lat:
+            lattice=lattice.mutate(degree=eps, frozen=True)
 
         h = splitter.H.number
         split_sites = []
@@ -758,7 +775,7 @@ class pyxtal:
                     dis /= np.linalg.norm(dis)
                     pos0 += eps*dis*(np.random.random()-0.5)
                     wp, _ = Wyckoff_position.from_symops(ops2, h, permutation=False)
-                    if h in [7, 14] and self.group.number == 31:
+                    if h in [5, 7, 8, 9, 12, 13, 14, 15] and self.group.number == 31:
                         diag = True
                     else:
                         diag = self.diag
@@ -775,7 +792,7 @@ class pyxtal:
                     pos0 -= np.floor(pos0)
                     dis = (np.random.sample(3) - 0.5).dot(self.lattice.matrix)
                     dis /= np.linalg.norm(dis)
-                    pos0 += eps*dis*(np.random.random()-0.5)
+                    pos0 += np.dot(eps*dis*(np.random.random()-0.5), self.lattice.inv_matrix)
                     wp, _ = Wyckoff_position.from_symops(ops2, h, permutation=False)
                     split_sites.append(atom_site(wp, pos0, site.specie))
 
@@ -783,7 +800,7 @@ class pyxtal:
             new_struc.numIons = [int(multiples*numIon) for numIon in self.numIons]
         new_struc.lattice = lattice
         new_struc.source = 'subgroup'
-        
+
         return new_struc
 
     def apply_perturbation(self, d_lat=0.05, d_coor=0.05, d_rot=1):
@@ -798,10 +815,10 @@ class pyxtal:
         self.lattice = self.lattice.mutate(degree=d_lat)
 
         if self.molecular:
-            for i, site in enumerate(self.mol_sites):
+            for site in self.mol_sites:
                 site.perturbate(lattice=self.lattice.matrix, trans=d_coor, rot=d_rot)
         else:
-            for i, site in enumerate(self.atom_sites):
+            for site in self.atom_sites:
                 site.perturbate(lattice=self.lattice.matrix, magnitude=d_coor)
 
         self.source = 'Perturbation'
@@ -814,7 +831,7 @@ class pyxtal:
 
     def _get_coords_and_species(self, absolute=False, unitcell=True):
         """
-        extract the coordinates and species information 
+        extract the coordinates and species information
 
         Args:
             abosulte: if True, return the cartesian coords otherwise fractional
@@ -842,7 +859,7 @@ class pyxtal:
 
             if absolute:
                 total_coords = total_coords.dot(self.lattice.matrix)
-        
+
         return total_coords, species
 
     def _get_formula(self):
@@ -857,7 +874,7 @@ class pyxtal:
         else:
             specie_list = []
             for site in self.atom_sites:
-                specie_list.extend([site.specie]*site.wp.multiplicity) 
+                specie_list.extend([site.specie]*site.wp.multiplicity)
             species = list(set(specie_list))
             numIons = np.zeros(len(species), dtype=int)
             for i, sp in enumerate(species):
@@ -873,7 +890,6 @@ class pyxtal:
         """
         export to ase Atoms object.
         """
-        from ase import Atoms
         if self.valid:
             if self.dim > 0:
                 lattice = self.lattice.copy()
@@ -900,7 +916,6 @@ class pyxtal:
         """
         export to Pymatgen structure object.
         """
-        from pymatgen.core.structure import Structure, Molecule
 
         if self.valid:
             if self.dim > 0:
@@ -939,13 +954,11 @@ class pyxtal:
         display the crystal structure
         """
         if self.molecular:
-            from pyxtal.viz import display_molecular
             return display_molecular(self, **kwargs)
         else:
-            from pyxtal.viz import display_atomic
             return display_atomic(self, **kwargs)
 
-    def optimize_lattice(self, iterations=3):
+    def optimize_lattice(self, iterations=3, force=False):
         """
         optimize the lattice if the cell has a bad inclination angles
         """
@@ -954,7 +967,7 @@ class pyxtal:
         for i in range(iterations):
             lattice, trans, opt = self.lattice.optimize()
             #print(self.lattice, "->", lattice)
-            if opt:
+            if force or opt:
                 if self.molecular:
                     sites = self.mol_sites
                 else:
@@ -962,33 +975,121 @@ class pyxtal:
 
                 for j, site in enumerate(sites):
                     count += 1
+                    #print("old lattice"); print(site.lattice); print(site.lattice.matrix)
                     pos_abs = np.dot(site.position, self.lattice.matrix)
                     pos_frac = pos_abs.dot(lattice.inv_matrix)
                     pos_frac -= np.floor(pos_frac)
-                    if self.molecular:
-                        site.lattice = lattice
+
                     # for P21/c, Pc, C2/c, check if opt the inclination angle
+                    #print(trans)
                     ops = site.wp.ops.copy()
-                    diag = False
-                    if self.group.number in [7, 14, 15]:
+                    if self.group.number in [5, 7, 8, 9, 12, 13, 14, 15]:
                         for k, op in enumerate(ops):
                             vec = op.translation_vector.dot(trans)
-                            #print(vec)
                             vec -= np.floor(vec)
                             op1 = op.from_rotation_and_translation(op.rotation_matrix, vec)
                             ops[k] = op1
-                        wp, perm = Wyckoff_position.from_symops(ops, self.group.number)            
-                    
-                        if not isinstance(perm, list): 
+                        wp, perm = Wyckoff_position.from_symops(ops, self.group.number)
+
+                        #print('perm', perm)
+                        if not isinstance(perm, list):
                             diag = True
                         else:
                             diag = False
+                            # maybe p21/a
                             pos_frac = pos_frac[perm]
-                        sites[j] = atom_site(wp, pos_frac, site.specie, diag)
-                        #print(sites[j].wp)
-                    #site.update()
+                        
+                        # from p21/n to p21/c
+                        #if self.diag and not diag:
+                        
+                        if self.molecular:
+                            ori = site.orientation
+                            #print("new lattice"); print(site.lattice); print(site.lattice.matrix)
+                            if site.diag and not diag:
+                                if perm == [2, 1, 0]:
+                                    #print("from p21/n to p21/a to p21/c")
+                                    lattice0 = lattice.swap_axis(ids=perm)
+                                    xyz, _ = site._get_coords_and_species(absolute=True, first=True)
+                                    xyz = np.dot(xyz, lattice.inv_matrix) #frac
+                                    xyz = np.dot(xyz[:, perm], lattice0.matrix)
+                                    #center = pos_frac.dot(lattice0.matrix)
+                                    center = site.molecule.get_center(xyz)
+                                    site.molecule.reset_positions(xyz-center)
+                                    ori.reset_matrix(np.eye(3))
 
-                self.lattice = lattice
+                                elif not np.allclose(lattice.matrix, np.triu(lattice.matrix)):
+                                    #print("from p21/n to p21/c")
+                                    site.lattice = lattice
+                                    xyz, _ = site._get_coords_and_species(absolute=False, first=True)
+                                    lattice0 = Lattice.from_matrix(lattice.matrix, 
+                                                                   ltype=lattice.ltype, 
+                                                                   reset=True)
+
+                                    xyz = np.dot(xyz, lattice0.matrix)
+                                    center = site.molecule.get_center(xyz)
+                                    site.molecule.reset_positions(xyz-center)
+                                    ori.reset_matrix(np.eye(3))
+                                else:
+                                    lattice0 = lattice
+                            #if not np.allclose(lattice.matrix, np.triu(lattice.matrix)):
+                            elif diag and not site.diag:
+                                #print("from p21/c to p21/n")
+                                site.lattice = lattice
+                                xyz, _ = site._get_coords_and_species(absolute=False, first=True)
+                                lattice0 = Lattice.from_matrix(lattice.matrix, 
+                                                          ltype=lattice.ltype, 
+                                                          reset=True)
+
+                                xyz = np.dot(xyz, lattice0.matrix)
+                                center = site.molecule.get_center(xyz)
+                                site.molecule.reset_positions(xyz-center)
+                                ori.reset_matrix(np.eye(3))
+                            elif not np.allclose(lattice.matrix, np.triu(lattice.matrix)):
+                                site.lattice = lattice
+                                xyz, _ = site._get_coords_and_species(absolute=False, first=True)
+                                lattice0 = Lattice.from_matrix(lattice.matrix, 
+                                                          ltype=lattice.ltype, 
+                                                          reset=True)
+
+                                xyz = np.dot(xyz, lattice0.matrix)
+                                center = site.molecule.get_center(xyz)
+                                site.molecule.reset_positions(xyz-center)
+                                ori.reset_matrix(np.eye(3))
+                            else:
+                                lattice0 = lattice
+                            #print("reset lattice"); print(lattice); print(lattice.matrix)
+                            #print("reset lattice"); print(lattice0); print(lattice0.matrix)
+                            #print(self.group.symbol, wp)
+                            mol = site.molecule
+                            sites[j] = mol_site(mol, pos_frac, ori, wp, lattice0, diag)
+                        else:
+                            sites[j] = atom_site(wp, pos_frac, site.specie, diag)
+                    else:
+                        diag = False
+                        #print(self.group.symbol)
+                        #print(lattice.matrix)
+                        if not np.allclose(lattice.matrix, np.triu(lattice.matrix)):
+                            site.lattice = lattice
+                            xyz, _ = sites[j]._get_coords_and_species(absolute=False, first=True)
+                            lattice0 = Lattice.from_matrix(lattice.matrix, 
+                                                      ltype=lattice.ltype, 
+                                                      reset=True)
+                            xyz = np.dot(xyz, lattice0.matrix)
+                        else:
+                            lattice0 = lattice
+                            xyz, _ = sites[j]._get_coords_and_species(absolute=True, first=True)
+                        #print(lattice0.matrix)
+ 
+                        center = sites[j].molecule.get_center(xyz)
+                        sites[j].molecule.reset_positions(xyz-center)
+                        sites[j].orientation.reset_matrix(np.eye(3))
+                        sites[j].position = pos_frac
+                        sites[j].lattice = lattice0
+
+                if self.molecular:
+                    self.lattice = lattice0
+                else:
+                    self.lattice = lattice
                 self.diag = diag
             else:
                 break
@@ -1043,7 +1144,7 @@ class pyxtal:
                  "dim": self.dim,
                  "valid": self.valid,
                 }
-       
+
         return dict0
 
     def load_dict(self, dict0):
@@ -1053,7 +1154,6 @@ class pyxtal:
         self.group = Group(dict0["group"])
         self.lattice = Lattice.from_matrix(dict0["lattice"], ltype=self.group.lattice_type)
         self.molecular = dict0["molecular"]
-        self.number = self.group.number
         self.factor = dict0["factor"]
         self.source = dict0["source"]
         self.dim = dict0["dim"]
@@ -1072,56 +1172,81 @@ class pyxtal:
                 sites.append(atom_site.load_dict(site))
             self.atom_sites = sites
 
-    def get_alternatives(self, key='permutation'):
+    def get_alternatives(self, include_self=True):
         """
         get alternative structure representations
 
         Args:
-            key: string, now supports only permutation
-
+            include_self (bool): return the original structure
         Return:
-            
+            list of structures
         """
-        new_strucs = []
-        # search if the 
-        res = self.group.get_alternatives()[key]
-        if len(res) > 0:
-            if key == 'permutation':
-                for swap in res:
-                    new_struc = self.copy()
-                    # swap lattice and atom_sites
-                    new_struc.lattice = self.lattice.swap_axis(ids=swap)
-                    # check if a shift is required
-                    for atom_site in new_struc.atom_sites:
-                        shift = atom_site.shift_by_swap(swap)
-                        if np.sum(shift) > 0:
-                            break
-                    # perform a swap by considering the shift
-                    for atom_site in new_struc.atom_sites:
-                        atom_site.swap_axis(swap, shift)
+        if include_self:
+            new_strucs = [self]
+        else:
+            new_strucs = []
 
-                    new_struc.source = key
-                    new_strucs.append(new_struc)
+        # the list of wyckoff indices in the original structure
+        # e.g. [0, 2, 2, 4] -> [a, c, c, e]
+        # ids = [len(self.group)-1-site.wp.index for site in self.atom_sites]
+
+        wyc_sets = self.group.get_alternatives()
+        No = len(wyc_sets['No.'])
+        if No > 1:
+            # skip the first setting since it is identity
+            for no in range(1,No):
+                new_struc, ids = self._get_alternative(wyc_sets, no)
+                new_strucs.append(new_struc)
         return new_strucs
 
-    def get_alternative(self, tran, indices):
+    def _get_alternative(self, wyc_set, index):
         """
         get alternative structure representations
 
         Args:
             tran: affine matrix
-            indices: the list of transformed wps
+            index: the list of transformed wps
 
         Returns:
             a new pyxtal structure after transformation
         """
         new_struc = self.copy()
-        new_struc.lattice = self.lattice.transform(tran[:3,:3])
-        for atom_site in new_struc.atom_sites:
-            atom_site.equivalent_set(tran, indices)
-        new_struc.numIons = int(np.linalg.det(tran[:3,:3]))*self.numIons
-        new_struc._get_formula()
 
-        new_struc.source = "New Wyckoff Set"
-        return new_struc
+        # xyz_string like 'x+1/4,y+1/4,z+1/4'
+        xyz_string = wyc_set['Coset Representative'][index]
+        op = get_inverse(SymmOp.from_xyz_string(xyz_string))
+        #op = SymmOp.from_xyz_string(xyz_string)
+
+        ids = []
+        for i, site in enumerate(new_struc.atom_sites):
+            id = len(self.group) - site.wp.index - 1
+            letter = wyc_set['Transformed WP'][index].split()[id]
+            ids.append(letters.index(letter))
+            wp = Wyckoff_position.from_group_and_index(self.group.number, letter)
+            pos = op.operate(site.position)
+            pos1 = search_matched_position(self.group, wp, pos)
+            if pos1 is not None:
+                new_struc.atom_sites[i] = atom_site(wp, pos1, site.specie)
+            else:
+                print(pos)
+                print(wp)
+                raise RuntimeError("Cannot find the right pos")
+
+        # switch lattice
+        R = op.affine_matrix[:3,:3] #rotation
+        matrix = np.dot(R, self.lattice.matrix)
+        new_struc.lattice = Lattice.from_matrix(matrix, ltype=self.group.lattice_type)
+        new_struc.source = "Alt. Wyckoff Set: " + xyz_string
+        return new_struc, ids
+
+    def check_distance(self):
+        """
+        check intermolecular distance for molecular crystal
+        """
+        if self.molecular:
+            for ms in self.mol_sites:
+                if not ms.check_distances():
+                    return False
+            return True
+
 
