@@ -1,241 +1,245 @@
 """Module for converting between hall and Hermann-Mauguin spacegroup numbers"""
-hall_numbers = [0, 1, 2,
-3,
-6,
-9,
-18,
-21,
-30,
-39,
-57,
-60,
-63,
-72,
-81,
-90,
-108,
-109,
-112,
-115,
-116,
-119,
-122,
-123,
-124,
-125,
-128,
-134,
-137,
-143,
-149,
-155,
-161,
-164,
-170,
-173,
-176,
-182,
-185,
-191,
-197,
-203,
-209,
-212,
-215,
-218,
-221,
-227,
-229,
-230,
-234,
-239,
-245,
-251,
-257,
-263,
-266,
-269,
-275,
-279,
-284,
-290,
-292,
-298,
-304,
-310,
-313,
-316,
-323,
-334,
-336,
-337,
-338,
-341,
-343,
-349,
-350,
-351,
-352,
-353,
-354,
-355,
-356,
-357,
-358,
-360,
-362,
-363,
-365,
-366,
-367,
-368,
-369,
-370,
-371,
-372,
-373,
-374,
-375,
-376,
-377,
-378,
-379,
-380,
-381,
-382,
-383,
-384,
-385,
-386,
-387,
-388,
-389,
-390,
-391,
-392,
-393,
-394,
-395,
-396,
-397,
-398,
-399,
-400,
-401,
-403,
-405,
-406,
-407,
-409,
-411,
-412,
-413,
-415,
-417,
-418,
-419,
-421,
-423,
-424,
-425,
-427,
-429,
-430,
-431,
-432,
-433,
-435,
-436,
-438,
-439,
-440,
-441,
-442,
-443,
-444,
-446,
-447,
-448,
-449,
-450,
-452,
-454,
-455,
-456,
-457,
-458,
-460,
-462,
-463,
-464,
-465,
-466,
-467,
-468,
-469,
-470,
-471,
-472,
-473,
-474,
-475,
-476,
-477,
-478,
-479,
-480,
-481,
-482,
-483,
-484,
-485,
-486,
-487,
-488,
-489,
-490,
-491,
-492,
-493,
-494,
-496,
-497,
-499,
-500,
-501,
-502,
-503,
-504,
-505,
-506,
-507,
-508,
-509,
-510,
-511,
-512,
-513,
-514,
-515,
-516,
-517,
-519,
-520,
-522,
-523,
-524,
-526,
-528,
-529,
-530]
-#array position (starting with 1) represent international #
-#Default Descriptions (consistent with cryst.ehu.eh):
-#unique axis b and cell choice 1 (b1 when there are a1, b1, c1)
-#origin choice 2
-#obverse setting triple hexagonal axes for R space groups
-#assumes unique axis b for (a or b or c) Hall symbols
+hall_numbers = [
+    0,
+    1,
+    2,
+    3,
+    6,
+    9,
+    18,
+    21,
+    30,
+    39,
+    57,
+    60,
+    63,
+    72,
+    81,
+    90,
+    108,
+    109,
+    112,
+    115,
+    116,
+    119,
+    122,
+    123,
+    124,
+    125,
+    128,
+    134,
+    137,
+    143,
+    149,
+    155,
+    161,
+    164,
+    170,
+    173,
+    176,
+    182,
+    185,
+    191,
+    197,
+    203,
+    209,
+    212,
+    215,
+    218,
+    221,
+    227,
+    229,
+    230,
+    234,
+    239,
+    245,
+    251,
+    257,
+    263,
+    266,
+    269,
+    275,
+    279,
+    284,
+    290,
+    292,
+    298,
+    304,
+    310,
+    313,
+    316,
+    323,
+    334,
+    336,
+    337,
+    338,
+    341,
+    343,
+    349,
+    350,
+    351,
+    352,
+    353,
+    354,
+    355,
+    356,
+    357,
+    358,
+    360,
+    362,
+    363,
+    365,
+    366,
+    367,
+    368,
+    369,
+    370,
+    371,
+    372,
+    373,
+    374,
+    375,
+    376,
+    377,
+    378,
+    379,
+    380,
+    381,
+    382,
+    383,
+    384,
+    385,
+    386,
+    387,
+    388,
+    389,
+    390,
+    391,
+    392,
+    393,
+    394,
+    395,
+    396,
+    397,
+    398,
+    399,
+    400,
+    401,
+    403,
+    405,
+    406,
+    407,
+    409,
+    411,
+    412,
+    413,
+    415,
+    417,
+    418,
+    419,
+    421,
+    423,
+    424,
+    425,
+    427,
+    429,
+    430,
+    431,
+    432,
+    433,
+    435,
+    436,
+    438,
+    439,
+    440,
+    441,
+    442,
+    443,
+    444,
+    446,
+    447,
+    448,
+    449,
+    450,
+    452,
+    454,
+    455,
+    456,
+    457,
+    458,
+    460,
+    462,
+    463,
+    464,
+    465,
+    466,
+    467,
+    468,
+    469,
+    470,
+    471,
+    472,
+    473,
+    474,
+    475,
+    476,
+    477,
+    478,
+    479,
+    480,
+    481,
+    482,
+    483,
+    484,
+    485,
+    486,
+    487,
+    488,
+    489,
+    490,
+    491,
+    492,
+    493,
+    494,
+    496,
+    497,
+    499,
+    500,
+    501,
+    502,
+    503,
+    504,
+    505,
+    506,
+    507,
+    508,
+    509,
+    510,
+    511,
+    512,
+    513,
+    514,
+    515,
+    516,
+    517,
+    519,
+    520,
+    522,
+    523,
+    524,
+    526,
+    528,
+    529,
+    530,
+]
+# array position (starting with 1) represent international #
+# Default Descriptions (consistent with cryst.ehu.eh):
+# unique axis b and cell choice 1 (b1 when there are a1, b1, c1)
+# origin choice 2
+# obverse setting triple hexagonal axes for R space groups
+# assumes unique axis b for (a or b or c) Hall symbols
 
-#TODO: incorporate non-standard setting transformations
+# TODO: incorporate non-standard setting transformations
 def hall_from_hm(num):
     """
     Returns a space group's Hall number from the international number
@@ -247,6 +251,8 @@ def hall_from_hm(num):
         the Hall number
     """
     return hall_numbers[num]
+
+
 """
 List of Hermann-Mauguin to Hall symbol conversions:
 HM #    HM Symbol      Hall Symbol(different from wyckoff.csv)
