@@ -9,11 +9,13 @@ constraints.
 
 """
 # Imports
-import numpy as np
+import os
 from copy import deepcopy
+from operator import itemgetter
+from random import choice
+import numpy as np
 from scipy.spatial.transform import Rotation
 import networkx as nx
-from random import choice
 # ------------------------------
 # External Libraries
 from pymatgen.core.structure import Molecule
@@ -31,6 +33,63 @@ from pyxtal.database.collection import Collection
 # ------------------------------
 molecule_collection = Collection("molecules")
 
+def cleaner(list_to_clean):
+    """
+    Remove duplicate torsion definion from a list of atom ind. tuples.
+    """
+    for_remove = []
+    for x in reversed(range(len(list_to_clean))):
+        for y in reversed(range(x)):
+            ix1, ix2 = itemgetter(1)(list_to_clean[x]), itemgetter(2)(list_to_clean[x])
+            iy1, iy2 = itemgetter(1)(list_to_clean[y]), itemgetter(2)(list_to_clean[y])
+            if (ix1 == iy1 and ix2 == iy2) or (ix1 == iy2 and ix2 == iy1):
+                for_remove.append(y)
+    clean_list = [v for i, v in enumerate(list_to_clean)
+                  if i not in set(for_remove)]
+    return clean_list
+
+def find_id_from_smile(smile):
+    """
+    Find the positions of rotatable bonds in the molecule.
+    """
+    from rdkit import Chem
+
+    smarts_torsion="[*]~[!$(*#*)&!D1]-&!@[!$(*#*)&!D1]~[*]"
+    mol = Chem.MolFromSmiles(smile)
+    pattern_tor = Chem.MolFromSmarts(smarts_torsion)
+    torsion = list(mol.GetSubstructMatches(pattern_tor))
+    return cleaner(torsion)
+
+def dihedral(p):
+    """
+    dihedral from https://stackoverflow.com/questions/20305272
+    """
+    p0 = p[0]
+    p1 = p[1]
+    p2 = p[2]
+    p3 = p[3]
+
+    b0 = -1.0*(p1 - p0)
+    b1 = p2 - p1
+    b2 = p3 - p2
+
+    # normalize b1 so that it does not influence magnitude of vector
+    # rejections that come next
+    b1 /= np.linalg.norm(b1)
+
+    # vector rejections
+    # v = projection of b0 onto plane perpendicular to b1
+    #   = b0 minus component that aligns with b1
+    # w = projection of b2 onto plane perpendicular to b1
+    #   = b2 minus component that aligns with b1
+    v = b0 - np.dot(b0, b1)*b1
+    w = b2 - np.dot(b2, b1)*b1
+
+    # angle between v and w in a plane is the torsion angle
+    # v and w may not be normalized but that's fine since tan is y/x
+    x = np.dot(v, w)
+    y = np.dot(np.cross(b1, v), w)
+    return np.degrees(np.arctan2(y, x))
 
 class pyxtal_molecule:
     """
@@ -38,18 +97,26 @@ class pyxtal_molecule:
     The added features include:
     0, parse the input
     1, estimate volume/tolerance/radii
-    2, find and store symmetry (todo)
-    3, get the principle axis (todo)
-    4, re-align the molecule (todo)
+    2, find and store symmetry 
+    3, get the principle axis 
+    4, re-align the molecule 
+
+    The molecule is always centered at (0, 0, 0).
+
+    If the smile format is used, the center is defined as in
+    https://www.rdkit.org/docs/source/rdkit.Chem.rdMolTransforms.html
+
+    Otherwise, the center is just the mean of atomic positions
 
     Args:
         mol: a string to reprent the molecule
         tm: tolerance matrix
-   
     """
 
     def __init__(self, mol=None, symmetrize=True, tm=Tol_matrix(prototype="molecular")):
         mo = None
+        self.smile = None
+        self.torsionlist = None
         if type(mol) == str:
             # Parse molecules: either file or molecule name
             tmp = mol.split(".")
@@ -57,11 +124,15 @@ class pyxtal_molecule:
             if len(tmp) > 1:
                 # Load the molecule from the given file
                 if tmp[-1] in ["xyz", "gjf", "g03", "json"]:
-                    import os
                     if os.path.exists(mol):
                         mo = Molecule.from_file(mol)
                     else:
                         raise NameError("{:s} is not a valid path".format(mol))
+                elif tmp[-1] == 'smi':
+                    self.smile = tmp[0]
+                    symbols, xyz, self.torsionlist = self.rdkit_mol_init(tmp[0])
+                    mo = Molecule(symbols, xyz)
+                    symmetrize = False
                 else:
                     raise NameError("{:s} is not a supported format".format(tmp[-1]))
             else:
@@ -80,7 +151,7 @@ class pyxtal_molecule:
         if len(mo) > 1:
             if symmetrize:
                 pga = PointGroupAnalyzer(mo)
-                mo = pga.symmetrize_molecule()["sym_mol"] 
+                mo = pga.symmetrize_molecule()["sym_mol"]
             mo = self.add_site_props(mo)
 
         self.mol = mo
@@ -90,11 +161,17 @@ class pyxtal_molecule:
         self.get_radius()
         self.get_symbols()
         self.get_tols_matrix()
+        xyz = self.mol.cart_coords
+        self.reset_positions(xyz-self.get_center(xyz))
+
 
     def __str__(self):
         return '[' + self.name + ']'
 
     def save_dict(self):
+        """
+        save the object as a dictionary
+        """
         return self.mol.as_dict()
 
     def copy(self):
@@ -103,19 +180,6 @@ class pyxtal_molecule:
         """
         return deepcopy(self)
 
-    def reset_positions(self, coors):
-        """
-        reset the coordinates
-        """
-        from pymatgen.core.sites import Site
-        if len(coors) != len(self.mol._sites):
-            raise ValueError("number of atoms is inconsistent!")
-        else:
-            for i, coor in enumerate(coors):
-                _site = self.mol._sites[i]
-                new_site = Site(_site.species, coor, properties=_site.properties)
-                self.mol._sites[i] = new_site
-
     @classmethod
     def load_dict(cls, dicts):
         """
@@ -135,6 +199,9 @@ class pyxtal_molecule:
         return pyxtal_molecule(mo, self.tm)
 
     def add_site_props(self, mo):
+        """
+        add site properties
+        """
         if len(self.props) > 0:
             for key in self.props.keys():
                 mo.add_site_property(key, self.props[key])
@@ -143,13 +210,13 @@ class pyxtal_molecule:
     def get_box(self):
         """
         Given a molecule, find a minimum orthorhombic box containing it.
-        Size is calculated using min and max x, y, and z values, 
+        Size is calculated using min and max x, y, and z values,
         plus the padding defined by the vdw radius
         For best results, call oriented_molecule first.
-        
+
         Args:
             mol: a pymatgen Molecule object. Should be oriented along its principle axes.
-    
+
         Returns:
             a Box object
         """
@@ -174,6 +241,9 @@ class pyxtal_molecule:
         self.axes = P
 
     def get_radius(self):
+        """
+        get the radius of a molecule
+        """
         r_max = 0
         for coord, number in zip(self.mol.cart_coords, self.mol.atomic_numbers):
             radius = (
@@ -189,6 +259,9 @@ class pyxtal_molecule:
             self.radius = rmin
 
     def has_stick_shape(self):
+        """
+        check if the molecule is stick-like
+        """
         sizes = [self.box.width,self.box.height,self.box.length]
         sizes.sort()
         if sizes[2]>15: #and sizes[2]/sizes[0]>2 and sizes[2]/sizes[1]>2:
@@ -208,16 +281,217 @@ class pyxtal_molecule:
         for i1, number1 in enumerate(numbers):
             for i2, number2 in enumerate(numbers):
                 tols[i1][i2] = self.tm.get_tol(number1, number2)
+                # allow hydrogen bond
+                if [number1, number2] in [[1,7], [1,8], [1,9], [7,1], [8,1], [9,1]]:
+                    tols[i1][i2] *= 0.9
         if len(self.mol)==1:
             tols *= 0.8 # if only one atom, reduce the tolerance
         self.tols_matrix = tols
 
     def show(self):
+        """
+        show the molecule
+        """
         from pyxtal.viz import display_molecules
         return display_molecules([self.mol])
 
 
+    def rdkit_mol_init(self, smile):
+        """
+        initialize the mol xyz and torsion list
+        """
+        from rdkit import Chem
+        from rdkit.Chem import AllChem
 
+        smarts_torsion="[*]~[!$(*#*)&!D1]-&!@[!$(*#*)&!D1]~[*]"
+        mol = Chem.MolFromSmiles(smile)
+        pattern_tor = Chem.MolFromSmarts(smarts_torsion)
+        torsion = list(mol.GetSubstructMatches(pattern_tor))
+        torsionlist = cleaner(torsion)
+
+        mol = Chem.AddHs(mol)
+        symbols = []
+        for id in range(mol.GetNumAtoms()):
+            symbols.append(mol.GetAtomWithIdx(id).GetSymbol())
+
+        AllChem.EmbedMultipleConfs(mol, numConfs=2, randomSeed=0xf00d)
+        conf = mol.GetConformer(0)
+        #print("Init: ", conf.GetPositions())
+        xyz = self.align(conf)
+        #print("Init: ", xyz[:3])
+        return symbols, xyz, torsionlist
+
+    def rdkit_mol(self, smile):
+        """
+        initialize the mol xyz and torsion list
+        """
+        from rdkit import Chem
+        from rdkit.Chem import AllChem
+
+        mol = Chem.MolFromSmiles(smile)
+        mol = Chem.AddHs(mol)
+        AllChem.EmbedMultipleConfs(mol, numConfs=3, randomSeed=0xf00d)
+        #print("To_dict", mol.GetConformer(0).GetPositions())
+        return mol
+
+    def align(self, conf, reflect=False):
+        """
+        Align the molecule and return the xyz
+        The default CanonicalizeConformer function may also include inversion
+        """
+        from rdkit.Chem import rdMolTransforms as rdmt
+
+        #rotation
+        #print("align: "); print(conf.GetPositions())
+        trans = rdmt.ComputeCanonicalTransform(conf)
+        if np.linalg.det(trans[:3,:3]) < 0:
+            trans[:3,:3] *= -1
+
+        if reflect:
+            trans[:3,:3] *= -1
+        #print(trans)
+        rdmt.TransformConformer(conf, trans)
+        
+        #print("rot", conf.GetPositions()[:3])
+        #translation
+        pt = rdmt.ComputeCentroid(conf)
+        center = np.array([pt.x, pt.y, pt.z])
+        xyz = conf.GetPositions() - center
+        #print("return", xyz[:3])
+        return xyz
+
+    def get_center(self, xyz):
+        """
+        get the molecular center for a transformed xyz
+        """
+        if self.smile is None:
+            return np.mean(xyz, axis=0)
+        else:
+            # from rdkit
+            from rdkit.Geometry import Point3D
+            from rdkit.Chem import rdMolTransforms as rdmt
+
+            conf1 = self.rdkit_mol(self.smile).GetConformer(0)
+            for i in range(conf1.GetNumAtoms()):
+                x, y, z = xyz[i]
+                conf1.SetAtomPosition(i, Point3D(x,y,z))
+            pt = rdmt.ComputeCentroid(conf1)
+            return np.array([pt.x, pt.y, pt.z])
+
+    def get_principle_axes(self, xyz):
+        """
+        get the principle axis for a rotated xyz
+        """
+        if self.smile is None:
+            Inertia = get_inertia_tensor(xyz)
+            _, matrix = np.linalg.eigh(Inertia)
+            return matrix
+
+        else:
+            from rdkit.Geometry import Point3D
+            from rdkit.Chem import rdMolTransforms as rdmt
+
+            conf1 = self.rdkit_mol(self.smile).GetConformer(0)
+            for i in range(len(self.mol)):
+                x,y,z = xyz[i]
+                conf1.SetAtomPosition(i,Point3D(x,y,z))
+
+            return rdmt.ComputePrincipalAxesAndMoments(conf1)
+
+    def get_torsion_angles(self, xyz):
+        """
+        get the torsion angles
+        """
+        from rdkit.Geometry import Point3D
+        from rdkit.Chem import rdMolTransforms as rdmt
+        mol = self.rdkit_mol(self.smile)
+        conf = mol.GetConformer(0)
+
+        for i in range(len(self.mol)):
+            x,y,z = xyz[i]
+            conf.SetAtomPosition(i,Point3D(x,y,z))
+
+        angs = []
+        for torsion in self.torsionlist:
+            (i, j, k, l) = torsion
+            angs.append(rdmt.GetDihedralDeg(conf, i, j, k, l))
+        return angs
+    
+    def set_torsion_angles(self, conf, angles, reflect=False):
+        """
+        reset the torsion angles and update molecular xyz
+        """
+        from rdkit.Chem import rdMolTransforms as rdmt
+
+        for id, torsion in enumerate(self.torsionlist):
+            (i, j, k, l) = torsion
+            rdmt.SetDihedralDeg(conf, i, j, k, l, angles[id])
+        
+        xyz = self.align(conf, reflect)
+        return xyz
+
+    def get_orientation(self, xyz, rtol=0.15):
+        """
+        get orientation, needs to check the tolerance
+        """
+        from rdkit.Geometry import Point3D
+        from rdkit.Chem import rdMolAlign, RemoveHs, rdmolfiles, rdMolTransforms
+        mol = self.rdkit_mol(self.smile)
+        
+        conf0 = mol.GetConformer(0)
+        conf1 = mol.GetConformer(1)
+        conf2 = mol.GetConformer(2)
+        angs = self.get_torsion_angles(xyz)
+        xyz0 = self.set_torsion_angles(conf0, angs) #conf0 with aligned
+        xyz1 = self.set_torsion_angles(conf0, angs, True) #conf0 with aligned
+
+        for i in range(len(self.mol)):
+            x0,y0,z0 = xyz0[i]
+            x1,y1,z1 = xyz1[i]
+            x,y,z = xyz[i]
+            conf0.SetAtomPosition(i,Point3D(x0,y0,z0))
+            conf1.SetAtomPosition(i,Point3D(x,y,z))
+            conf2.SetAtomPosition(i,Point3D(x1,y1,z1))
+
+        mol = RemoveHs(mol)
+        rmsd1, trans1 = rdMolAlign.GetAlignmentTransform(mol, mol, 1, 0)
+        rmsd2, trans2 = rdMolAlign.GetAlignmentTransform(mol, mol, 1, 2)
+        tol = rtol*mol.GetNumAtoms()
+
+        #print(rmsd1, rmsd2)
+        if rmsd1 < tol:
+            trans = trans1[:3,:3].T
+            r = Rotation.from_matrix(trans)
+            return r.as_euler('zxy', degrees=True), rmsd1, False
+        elif rmsd2 < tol:
+            trans = trans2[:3,:3].T
+            r = Rotation.from_matrix(trans)
+            return r.as_euler('zxy', degrees=True), rmsd2, True
+        else:
+            print(rmsd1, rmsd2)
+            #rdmolfiles.MolToXYZFile(mol, '1.xyz', 0)
+            #rdmolfiles.MolToXYZFile(mol, '2.xyz', 1)
+            #rdmolfiles.MolToXYZFile(mol, '3.xyz', 2)
+            print(self.get_torsion_angles(xyz))   
+            print(self.get_torsion_angles(xyz0))   
+            print(self.get_torsion_angles(xyz1))   
+            raise ValueError("Problem in conformer")
+
+
+    def reset_positions(self, coors):
+        """
+        reset the coordinates
+        """
+        from pymatgen.core.sites import Site
+        if len(coors) != len(self.mol._sites):
+            raise ValueError("number of atoms is inconsistent!")
+        else:
+            for i, coor in enumerate(coors):
+                _site = self.mol._sites[i]
+                new_site = Site(_site.species, coor, properties=_site.properties)
+                self.mol._sites[i] = new_site
+
+      
 
 class Box:
     """
@@ -316,7 +590,7 @@ class Orientation:
                  "axis": self.axis
                 }
         return dict0
-    
+
     @classmethod
     def load_dict(cls, dicts):
         matrix = dicts['matrix']
@@ -330,9 +604,9 @@ class Orientation:
         rotate about the constraint axis.
 
         Args:
-            angle: an angle to rotate about the constraint axis. 
-            If "random", chooses a random rotation angle. 
-            If self.degrees==2, chooses a random rotation matrix. 
+            angle: an angle to rotate about the constraint axis.
+            If "random", chooses a random rotation angle.
+            If self.degrees==2, chooses a random rotation matrix.
             If self.degrees==1, only apply on angle
             If self.degrees==0, no change
 
@@ -340,14 +614,14 @@ class Orientation:
         if self.degrees >= 1:
             # choose the axis
             if self.axis is None:
-                axis = np.random.RandomState().rand(3) - 0.5
+                axis = np.random.rand(3) - 0.5
                 self.axis = axis / np.linalg.norm(axis)
- 
+
             # parse the angle
             if angle == "random":
-                angle = np.random.RandomState().rand() * np.pi * 2
+                angle = np.random.rand() * np.pi * 2
             self.angle = angle
-    
+
             # update the matrix
             r1 = Rotation.from_rotvec(self.angle * self.axis)
 
@@ -358,7 +632,6 @@ class Orientation:
                     r2 = Rotation.from_euler(ax, angle0, degrees=True)
                     r1 = r2*r1
             self.r = r1 * self.r
-            #self.r *= r1 
             self.matrix = self.r.as_matrix()
 
     def rotate_by_matrix(self, matrix, ignore_constraint=True):
@@ -379,7 +652,7 @@ class Orientation:
                     raise ValueError("must rotate along the given axis")
         else:
             axis = None
-            
+
         matrix = matrix.dot(self.matrix)
         return Orientation(matrix, self.degrees, axis)
 
@@ -417,7 +690,7 @@ class Orientation:
         elif self.degrees == 0:
             return self.matrix
 
-    def get_op(self, angle=None):
+    def get_op(self): #, angle=None):
         """
         Generate a SymmOp object consistent with the orientation's
         constraints. Allows for specification of an angle (possibly random) to
@@ -436,57 +709,6 @@ class Orientation:
         #    self.change_orientation(angle)
         return SymmOp.from_rotation_and_translation(self.matrix, [0, 0, 0])
 
-    @classmethod
-    def from_constraint(self, v1, c1):
-        """
-        Geneate an orientation object given a constraint axis c1, and a
-        corresponding vector v1. v1 will be rotated onto c1, and the resulting
-        orientation will have a rotational degree of freedom about c1.
-
-        Args:
-            v1: a 1x3 vector in the original reference frame
-            c1: a corresponding axis which v1 must be mapped to
-
-        Returns:
-            an orientation object consistent with the supplied constraint
-        """
-        # c1 is the constraint vector; v1 will be rotated onto it
-        m = rotate_vector(v1, c1)
-        return Orientation(m, degrees=1, axis=c1)
-
-    @classmethod
-    def from_constraints(self, v1, c1, v2, c2):
-        """
-        Geneate an orientation object given two constraint vectors
-
-        Args:
-            v1: a 1x3 vector in the original reference frame
-            c1: a corresponding axis which v1 must be mapped to
-            v1: a second 1x3 vector in the original reference frame
-            c1: a corresponding axis which v2 must be mapped to
-
-        Returns:
-            an orientation object consistent with the supplied constraints
-        """
-        T = rotate_vector(v1, c1)
-        phi = angle(c1, c2)
-        phi2 = angle(c1, (np.dot(T, v2)))
-        if not np.isclose(phi, phi2, rtol=0.01):
-            printx("Error: constraints and vectors do not match.", priority=1)
-            return
-        r = np.sin(phi)
-        c = np.linalg.norm(np.dot(T, v2) - c2)
-        theta = np.arccos(1 - (c ** 2) / (2 * (r ** 2)))
-        Rot = R.from_rotvec(theta * c1)
-        T2 = np.dot(R, T)
-        a = angle(np.dot(T2, v2), c2)
-        if not np.isclose(a, 0, rtol=0.01):
-            T2 = np.dot(np.linalg.inv(R), T)
-        a = angle(np.dot(T2, v2), c2)
-        if not np.isclose(a, 0, rtol=0.01):
-            printx("Error: Generated incorrect rotation: " + str(theta), priority=1)
-        return Orientation(T2, degrees=0)
-
     def random_orientation(self):
         """
         Applies random rotation (if possible) and returns a new orientation with
@@ -500,12 +722,15 @@ class Orientation:
         return self
 
     def get_Euler_angles(self):
+        """
+        get the Euler angles
+        """
         return self.r.as_euler('zxy', degrees=True)
 
 
 def get_inertia_tensor(coords):
     """
-    Calculate the symmetric inertia tensor for a Molecule 
+    Calculate the symmetric inertia tensor for a Molecule
     the principal axes of symmetry.
 
     Args:
@@ -522,22 +747,20 @@ def get_inertia_tensor(coords):
     Inertia[0,1] = Inertia[1,0] = -np.sum(coords[:,0]*coords[:,1])
     Inertia[0,2] = Inertia[2,0] = -np.sum(coords[:,0]*coords[:,2])
     Inertia[1,2] = Inertia[2,1] = -np.sum(coords[:,1]*coords[:,2])
- 
+
     return Inertia
 
 
-def reoriented_molecule(mol, nested=False):
+def reoriented_molecule(mol): #, nested=False):
     """
     Reorient a molecule so that its principal axes are aligned with the
     identity matrix.
 
     Args:
         mol: a Molecule object
-        nested: keep track of how many times the function
-            has been called recursively
 
     Returns:
-        new_mol: a reoriented copy of the original molecule. 
+        new_mol: a reoriented copy of the original molecule.
         P: the 3x3 rotation matrix used to obtain it.
     """
     coords = mol.cart_coords
@@ -572,7 +795,7 @@ def get_symmetry(mol, already_oriented=False):
     pga = PointGroupAnalyzer(mol)
     # Handle linear molecules
     if "*" in pga.sch_symbol:
-        if already_oriented == False:
+        if not already_oriented:
             # Reorient the molecule
             oriented_mol, P = reoriented_molecule(mol)
             pga = PointGroupAnalyzer(oriented_mol)
@@ -594,15 +817,15 @@ def get_symmetry(mol, already_oriented=False):
                 if axis == [1, 0, 0]:
                     symm_m.append(SymmOp.from_xyz_string("x,-y,z"))
                     symm_m.append(SymmOp.from_xyz_string("x,y,-z"))
-                    r = SymmOp.from_xyz_string("-x,y,-z")
+                    #r = SymmOp.from_xyz_string("-x,y,-z")
                 elif axis == [0, 1, 0]:
                     symm_m.append(SymmOp.from_xyz_string("-x,y,z"))
                     symm_m.append(SymmOp.from_xyz_string("x,y,-z"))
-                    r = SymmOp.from_xyz_string("-x,-y,z")
+                    #r = SymmOp.from_xyz_string("-x,-y,z")
                 elif axis == [0, 0, 1]:
                     symm_m.append(SymmOp.from_xyz_string("-x,y,z"))
                     symm_m.append(SymmOp.from_xyz_string("x,-y,z"))
-                    r = SymmOp.from_xyz_string("x,-y,-z")
+                    #r = SymmOp.from_xyz_string("x,-y,-z")
                 # Generate a full list of SymmOps for the molecule's pointgroup
                 symm_m = generate_full_symmops(symm_m, 1e-3)
                 break
@@ -663,7 +886,6 @@ def orientation_in_wyckoff_position(
 
     wyckoffs = wyckoff_position.ops
     w_symm = wyckoff_position.symmetry_m
-    index = wyckoff_position.index
 
     # Obtain the Wyckoff symmetry
     symm_w = w_symm[0]
@@ -758,7 +980,7 @@ def orientation_in_wyckoff_position(
                     # Check if axes are symmetrically equivalent
                     else:
                         cond1 = False
-                        cond2 = False
+                        # cond2 = False
                         for opa in opa_m:
                             if opa.type == "rotation":
                                 op = opa.op
@@ -867,14 +1089,16 @@ def orientation_in_wyckoff_position(
     allowed = []
     for o in orientations_new:
         if randomize is True:
-            op = o.get_op("random")
+            op = o.get_op()
         elif randomize is False:
             op = o.get_op() #do not change
         mo = deepcopy(mol)
         mo.apply_operation(op)
-        if orientation_in_wyckoff_position(mo, wyckoff_position, exact_orientation=True, randomize=False, allow_inversion=allow_inversion) is True:
+        if orientation_in_wyckoff_position(
+                mo, wyckoff_position, exact_orientation=True,
+                randomize=False, allow_inversion=allow_inversion
+        ):
             allowed.append(o)
-    #Return the array of allowed orientations. If there are none, return False
     if allowed == []:
         return False
     else:
@@ -883,7 +1107,7 @@ def orientation_in_wyckoff_position(
 def make_graph(mol, tol=0.2):
     """
     make graph object for the input molecule
-    """   
+    """
     G = nx.Graph()
     names = {}
     for i, site in enumerate(mol._sites):
@@ -911,7 +1135,7 @@ def make_graph(mol, tol=0.2):
     nx.set_node_attributes(G, names, 'name')
 
     return G
- 
+
 def compare_mol_connectivity(mol1, mol2, ignore_name=False):
     """
     Compare two molecules by connectivity
@@ -926,5 +1150,3 @@ def compare_mol_connectivity(mol1, mol2, ignore_name=False):
         GM = nx.isomorphism.GraphMatcher(G1, G2, node_match=fun)
 
     return GM.is_isomorphic(), GM.mapping
-
-
