@@ -29,9 +29,8 @@ from pyxtal.operations import (
 )
 from pyxtal.database.element import Element
 from pyxtal.database.hall import hall_from_hm
-
+from pyxtal.constants import letters
 # ------------------------------ Constants ---------------------------------------
-letters = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
 
 wyckoff_df = read_csv(resource_filename("pyxtal", "database/wyckoff_list.csv"))
 wyckoff_symmetry_df = read_csv(
@@ -57,7 +56,9 @@ symbols = loadfn(resource_filename("pyxtal", "database/symbols.json"))
 
 t_subgroup = loadfn(resource_filename("pyxtal",'database/t_subgroup.json'))
 k_subgroup = loadfn(resource_filename("pyxtal",'database/k_subgroup.json'))
+wyc_sets = loadfn(resource_filename("pyxtal",'database/wyckoff_sets.json'))
 
+t2h = SymmOp.from_rotation_and_translation([[1, -0.5, 0], [0, np.sqrt(3) / 2, 0], [0, 0, 1]], [0, 0, 0])
 Identity = SymmOp.from_xyz_string("x,y,z")
 Inversion = SymmOp.from_xyz_string("-x,-y,-z")
 op_o = SymmOp.from_xyz_string("0,0,0")
@@ -147,14 +148,16 @@ class Group:
         # Wyckoff positions, site_symmetry, generators, inverse
         # QZ: check if we can just use the get_wyckoff_symmetry function
         if dim == 3:
-            if self.number in [7, 14, 15]:
+            if self.number in [5, 7, 8, 9, 12, 13, 14, 15]:
                 self.alias = self.symbol.replace("c","n")
+                self.alias = self.alias.replace("C","I")
             self.wyckoffs = get_wyckoffs(self.number) 
             self.w_symm = get_wyckoff_symmetry(self.number)
             self.wyckoff_generators = get_wyckoff_generators(self.number)
             self.w_symm_m = get_wyckoff_symmetry(self.number, molecular=True)
             self.wyckoff_generators_m = get_wyckoff_generators(self.number, molecular=True) 
             self.hall_number = hall_from_hm(self.number)
+            self.point_group = get_point_group(self.number)
         elif dim == 2:
             self.wyckoffs = get_layer(self.number)
             self.w_symm = get_layer_symmetry(self.number)
@@ -261,10 +264,10 @@ class Group:
             True or False
         """
         # remove the multiplicity:
-        for site in sites:
+        for i, site in enumerate(sites):
             if len(site) > 1:
-                site = site[-1]
-
+                sites[i] = site[-1]
+        
         for wp in self:
             letter = wp.letter
             if sites.count(letter)>1:
@@ -440,29 +443,16 @@ class Group:
             return t_subgroup[str(self.number)]
         else:
             raise NotImplementedError("Now we only support the subgroups for space group")
-
+    
     def get_alternatives(self):
         """
         Get the alternative settings as a dictionary
         """
-        dicts = {"permutation": [],
-                 #"origin": [],
-                 #"diagonal": [],
-                }
         if self.dim == 3:
-            if self.number in [17, 18, 25, 27, 34, 35, 
-                               37, 38, 42, 43, 44, 49, 
-                               51, 52, 54, 56, 58, 59, 
-                               68, 74]:
-                dicts["permutation"] = [[1,0,2]] # like Pmm2
-            elif self.number in [16, 19, 21, 22, 23, 24, 
-                                 47, 48, 65, 69, 70, 71]:
-                dicts["permutation"] = [[1,0,2],[1,2,0],[0,2,1],[2,1,0],[2,0,1]] # like Pmmm
+            return wyc_sets[str(self.number)]
         else:
             raise NotImplementedError("Now we only support the subgroups for space group")
 
-        return dicts
-
     def get_max_k_subgroup(self):
         """
         Returns the maximal k-subgroups as a dictionary
@@ -472,7 +462,16 @@ class Group:
         else:
             raise NotImplementedError("Now we only support the subgroups for space group")
 
-    def get_min_supergroup(self, group_type='t'):
+    def get_max_t_subgroup(self):
+        """
+        Returns the maximal t-subgroups as a dictionary
+        """
+        if self.dim == 3:
+            return t_subgroup[str(self.number)]
+        else:
+            raise NotImplementedError("Now we only support the subgroups for space group")
+ 
+    def get_min_supergroup(self, group_type='t', G=None):
         """
         Returns the minimal supergroups as a dictionary
         """
@@ -482,14 +481,19 @@ class Group:
                      'relations': [],
                      'idx': [],
                     }
-            for sg in range(1, 231):
+            if G is None:
+                sgs = range(1,231)
+            else:
+                sgs = G
+
+            for sg in sgs:
                 subgroups = None
                 if group_type == 't':
                     if sg>self.number:
                         subgroups = Group(sg).get_max_t_subgroup()
                 else:
                     g1 = Group(sg)
-                    if g1.lattice_type == self.lattice_type:
+                    if g1.point_group == self.point_group:
                         subgroups = Group(sg).get_max_k_subgroup()
                 if subgroups is not None:
                     for i, sub in enumerate(subgroups['subgroup']): 
@@ -504,6 +508,19 @@ class Group:
         else:
             raise NotImplementedError("Now we only support the supergroups for space group")
 
+    def get_max_subgroup_numbers(self):
+        """
+        Returns the minimal supergroups as a dictionary
+        """
+        groups = []
+        if self.dim == 3:
+            k = k_subgroup[str(self.number)]['subgroup']
+            t = t_subgroup[str(self.number)]['subgroup']
+            return k+t
+        else:
+            raise NotImplementedError("Now we only support the supergroups for space group")
+
+
     @classmethod
     def list_groups(cls, dim=3):
         """
@@ -584,16 +601,26 @@ class Wyckoff_position:
 
     def diagonalize_symops(self):
         """
-        Obtain the symmetry in n representation for P21/c, Pc, C2/c
+        Obtain the symmetry in n representation for P2/c, Cc, P21/c, Pc, C2/c
         """
         ops = Group(self.number)[self.index]
-        if self.number in [7, 14, 15]:
+        #Pn, Cn, P2/n, P21/n, C2/n
+        if self.number in [7, 9, 13, 14, 15]:
             trans = np.array([[1,0,0],[0,1,0],[1,0,1]])
             for j, op in enumerate(ops):
                 vec = op.translation_vector.dot(trans)
                 vec -= np.floor(vec) 
                 op1 = op.from_rotation_and_translation(op.rotation_matrix, vec)
                 self.ops[j] = op1    
+        #I2, Im, Ic, I2/m, I2/c
+        elif self.number in [5, 8, 9, 12, 15]:
+            trans = np.array([[1,0,1],[0,1,0],[1,0,1]])
+            for j, op in enumerate(ops):
+                vec = op.translation_vector.dot(trans)
+                vec -= np.floor(vec) 
+                op1 = op.from_rotation_and_translation(op.rotation_matrix, vec)
+                self.ops[j] = op1  
+        #In, I2/n
 
     def equivalent_set(self, index):
         """
@@ -697,7 +724,7 @@ class Wyckoff_position:
                             return wyc, perm
 
                     # Try monoclinic space groups (P21/n, Pn, C2/n) 
-                    if i in [7, 14, 15]:
+                    if i in [7, 9, 13, 14, 15]:
                         trans = np.array([[1,0,0],[0,1,0],[1,0,1]])
                         str3 = []
                         for j, op in enumerate(wyc.ops):
@@ -708,6 +735,17 @@ class Wyckoff_position:
                             #wyc.ops[j] = op3
                         if set(str3) == set(str1):
                             return wyc, trans
+                    elif i in [5, 8, 9, 12, 15]:
+                        trans = np.array([[1,0,1],[0,1,0],[1,0,1]])
+                        str3 = []
+                        for j, op in enumerate(wyc.ops):
+                            vec = op.translation_vector.dot(trans)
+                            vec -= np.floor(vec) 
+                            op3 = op.from_rotation_and_translation(op.rotation_matrix, vec)
+                            str3.append(op3.as_xyz_string().replace("-1/2","+1/2"))
+                            #wyc.ops[j] = op3
+                        if set(str3) == set(str1):
+                            return wyc, trans
                 elif len(wyc) < N_sym:
                     continue #break since it won'
 
@@ -1832,6 +1870,7 @@ def get_point(num, organized=False):
 
 
 def get_wyckoff_symmetry(sg, PBC=[1, 1, 1], molecular=False):
+    #QZ: to get rid of something
     """
     Returns a list of Wyckoff position site symmetry for a given space group.
     1st index: index of WP in sg (0 is the WP with largest multiplicity)
@@ -1859,15 +1898,11 @@ def get_wyckoff_symmetry(sg, PBC=[1, 1, 1], molecular=False):
         coor = np.array(coor)
     wyckoffs = get_wyckoffs(sg, PBC=PBC)
 
-    P = SymmOp.from_rotation_and_translation(
-        [[1, -0.5, 0], [0, np.sqrt(3) / 2, 0], [0, 0, 1]], [0, 0, 0]
-    )
+
     symmetry_strings = eval(wyckoff_symmetry_df["0"][sg])
     symmetry = []
     convert = False
-    if molecular is True:
-        if sg >= 143 and sg <= 194:
-            convert = True
+    if molecular and 143 <= sg <= 194: convert = True
     # Loop over Wyckoff positions
     for x, w in zip(symmetry_strings, wyckoffs):
         if PBC != [1, 1, 1]:
@@ -1878,7 +1913,7 @@ def get_wyckoff_symmetry(sg, PBC=[1, 1, 1], molecular=False):
                 if not a:
                     if not abs(coor1[i] - 0.5) < 1e-2:
                         invalid = True
-            if invalid == False:
+            if not invalid:
                 symmetry.append([])
                 # Loop over points in WP
                 for y in x:
@@ -1886,12 +1921,11 @@ def get_wyckoff_symmetry(sg, PBC=[1, 1, 1], molecular=False):
                     # Loop over ops
                     for z in y:
                         op = SymmOp.from_xyz_string(z)
-                        if convert is True:
-                            # Convert non-orthogonal trigonal/hexagonal operations
-                            op = P * op * P.inverse
-                        if molecular is False:
+                        # Convert non-orthogonal trigonal/hexagonal operations
+                        if convert: op = t2h * op * t2h.inverse
+                        if not molecular:
                             symmetry[-1][-1].append(op)
-                        elif molecular is True:
+                        else:
                             op = SymmOp.from_rotation_and_translation(
                                 op.rotation_matrix, [0, 0, 0]
                             )
@@ -1904,12 +1938,12 @@ def get_wyckoff_symmetry(sg, PBC=[1, 1, 1], molecular=False):
                 # Loop over ops
                 for z in y:
                     op = SymmOp.from_xyz_string(z)
-                    if convert is True:
+                    if convert:
                         # Convert non-orthogonal trigonal/hexagonal operations
-                        op = P * op * P.inverse
-                    if molecular is False:
+                        op = t2h * op * t2h.inverse
+                    if not molecular:
                         symmetry[-1][-1].append(op)
-                    elif molecular is True:
+                    else:
                         op = SymmOp.from_rotation_and_translation(
                             op.rotation_matrix, [0, 0, 0]
                         )
@@ -1937,15 +1971,10 @@ def get_layer_symmetry(num, molecular=False):
         point in each Wyckoff position
     """
 
-    P = SymmOp.from_rotation_and_translation(
-        [[1, -0.5, 0], [0, np.sqrt(3) / 2, 0], [0, 0, 1]], [0, 0, 0]
-    )
     symmetry_strings = eval(layer_symmetry_df["0"][num])
     symmetry = []
     convert = False
-    if molecular is True:
-        if num >= 65:
-            convert = True
+    if molecular and num >= 65: convert = True
     # Loop over Wyckoff positions
     for x in symmetry_strings:
         symmetry.append([])
@@ -1955,12 +1984,12 @@ def get_layer_symmetry(num, molecular=False):
             # Loop over ops
             for z in y:
                 op = SymmOp.from_xyz_string(z)
-                if convert is True:
+                if convert:
                     # Convert non-orthogonal trigonal/hexagonal operations
-                    op = P * op * P.inverse
-                if molecular is False:
+                    op = t2h * op * t2h.inverse
+                if not molecular:
                     symmetry[-1][-1].append(op)
-                elif molecular is True:
+                else:
                     op = SymmOp.from_rotation_and_translation(
                         op.rotation_matrix, [0, 0, 0]
                     )
@@ -1988,15 +2017,10 @@ def get_rod_symmetry(num, molecular=False):
         point in each Wyckoff position
     """
 
-    P = SymmOp.from_rotation_and_translation(
-        [[1, -0.5, 0], [0, np.sqrt(3) / 2, 0], [0, 0, 1]], [0, 0, 0]
-    )
     symmetry_strings = eval(rod_symmetry_df["0"][num])
     symmetry = []
     convert = False
-    if molecular is True:
-        if num >= 42:
-            convert = True
+    if molecular and num >= 42: convert = True
     # Loop over Wyckoff positions
     for x in symmetry_strings:
         symmetry.append([])
@@ -2006,12 +2030,11 @@ def get_rod_symmetry(num, molecular=False):
             # Loop over ops
             for z in y:
                 op = SymmOp.from_xyz_string(z)
-                if convert is True:
                     # Convert non-orthogonal trigonal/hexagonal operations
-                    op = P * op * P.inverse
-                if molecular is False:
+                if convert: op = t2h * op * t2h.inverse
+                if not molecular:
                     symmetry[-1][-1].append(op)
-                elif molecular is True:
+                else:
                     op = SymmOp.from_rotation_and_translation(
                         op.rotation_matrix, [0, 0, 0]
                     )
@@ -2530,3 +2553,190 @@ def get_pbc_and_lattice(number, dim):
             lattice_type = "ellipsoidal"
     return PBC, lattice_type
 
+def search_matched_position(G, wp, pos):
+    """
+    search generator for a special Wyckoff position
+
+    Args:
+        G: space group number or Group object
+        wp: Wyckoff object
+        pos: initial xyz position
+
+    Return:
+        pos1: the position that matchs the standard setting
+    """
+    wp0 = G[0]
+    match = False
+
+    for op in wp0:
+        pos1 = op.operate(pos)
+        pos0 = wp[0].operate(pos1)
+        diff = pos1 - pos0
+        diff -= np.round(diff)
+        diff = np.abs(diff)
+        #print(wp.letter, pos1, pos0, diff)
+        if diff.sum()<1e-2:
+            pos1 -= np.floor(pos1)
+            match = True
+            break
+    if match:
+        return pos1
+    else:
+        return None
+
+def search_matched_positions(G, wp, pos):
+    """
+    search generator for a special Wyckoff position
+
+    Args:
+        G: space group number or Group object
+        wp: Wyckoff object
+        pos: initial xyz position
+
+    Return:
+        pos1: the position that matchs the standard setting
+    """
+    wp0 = G[0]
+    coords = []
+
+    for op in wp0:
+        pos1 = op.operate(pos)
+        pos0 = wp[0].operate(pos1)
+        diff = pos1 - pos0
+        diff -= np.round(diff)
+        diff = np.abs(diff)
+        #print(wp.letter, pos1, pos0, diff)
+        if diff.sum()<1e-2:
+            pos1 -= np.floor(pos1)
+            coords.append(pos1)
+    return coords
+
+def search_cloest_wp(G, wp, op, pos):
+    """
+    For a given position, search for the cloest wp which
+    satisfies the desired symmetry relation
+    e.g., for pos (0.1, 0.12, 0.2) and op (x, x, z)
+    the closest match is (0.11, 0.11, 0.2)
+
+    Args:
+        G: space group number or Group object
+        wp: Wyckoff object
+        op: symmetry operation belonging to wp 
+        pos: initial xyz position
+
+    Return:
+        pos1: the position that matchs symmetry operation
+    """
+    #G = Group(wp.number)
+    if np.linalg.matrix_rank(op.rotation_matrix) == 0:
+        # fixed point (e.g, 1/2, 1/2, 1/2)
+        return op.translation_vector
+    elif np.linalg.matrix_rank(op.rotation_matrix) == 3:
+        # fully independent, e.g., (x,y,z), (-x,y,z)
+        return pos
+    else:
+        # check if this is already matched
+        coords = search_matched_positions(G, wp, pos)
+        if len(coords)>0:
+            diffs = []
+            for coord in coords:
+                tmp = op.operate(coord)
+                diff1 = tmp - pos
+                diff1 -= np.round(diff1)
+                dist = np.linalg.norm(diff1) 
+                if dist < 1e-3:
+                    return tmp
+                else:
+                    diffs.append(dist)
+            minID = np.argmin(diffs)
+            return op.operate(coords[minID])
+
+        # if not match, search for the closest solution
+        else:
+            wp0 = G[0]
+            # extract all possible xyzs
+            all_xyz = apply_ops(pos, wp0)[1:]
+            dists = all_xyz - pos
+            dists -= np.round(dists)
+            ds = np.linalg.norm(dists, axis=1)
+            ids = np.argsort(ds)
+            for id in ids:
+                d = all_xyz[id] - pos
+                d -= np.round(d)
+                res = pos + d/2
+                if search_matched_position(G, wp, res) is not None:
+                    #print(ds[id], pos, res)
+                    return res
+            return op.operate(pos)
+
+
+def get_point_group(number):
+    """
+    return the point group for the given space group
+    """
+
+    if number == 1:
+        return '1'
+    elif number == 2:
+        return '-1'
+    elif 3 <= number <= 5:
+        return '2'
+    elif 6 <= number <= 9:
+        return 'm'
+    elif 10 <= number <= 15:
+        return '2/m'
+    elif 16 <= number <= 24:
+        return '222'
+    elif 25 <= number <= 46:
+        return 'mm2'
+    elif 47 <= number <= 74:
+        return 'mmm'
+    elif 75 <= number <= 80:
+        return '4'
+    elif 81 <= number <= 82:
+        return '-4'
+    elif 83 <= number <= 88:
+        return '4/m'
+    elif 89 <= number <= 98:
+        return '422'
+    elif 99 <= number <= 110:
+        return '4mm'
+    elif 111 <= number <= 122:
+        return '-42m'
+    elif 123 <= number <= 142:
+        return '4/mmm'
+    elif 143 <= number <= 146:
+        return '3'
+    elif 147 <= number <= 148:
+        return '-3'
+    elif 149 <= number <= 155:
+        return '32'
+    elif 156 <= number <= 161:
+        return '3m'
+    elif 162 <= number <= 167:
+        return '-3m'
+    elif 168 <= number <= 173:
+        return '6'
+    elif number == 174:
+        return '-6'
+    elif 175 <= number <= 176:
+        return '6/m'
+    elif 177 <= number <= 182:
+        return '622'
+    elif 183 <= number <= 186:
+        return '6mm'
+    elif 187 <= number <= 190:
+        return '-6m2'
+    elif 191 <= number <= 194:
+        return '6/mmm'
+    elif 195 <= number <= 199:
+        return '23'
+    elif 200 <= number <= 206:
+        return 'm-3'
+    elif 207 <= number <= 214:
+        return '432'
+    elif 215 <= number <= 220:
+        return '-43m'
+    elif 221 <= number <= 230:
+        return 'm-3m'
+
