@@ -1,13 +1,15 @@
 # python -m unittest pyxtal/test_all.py
 import unittest
 
+from random import choice
 import numpy as np
 from pkg_resources import resource_filename
+from pymatgen.core import Structure
+from pymatgen.core import Lattice as pmg_Lattice
 from pymatgen.core.structure import Molecule
 import pymatgen.analysis.structure_matcher as sm
 from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
 from pymatgen.core.operations import SymmOp
-from pymatgen import Structure
 
 from pyxtal import pyxtal
 from pyxtal.lattice import Lattice
@@ -53,6 +55,44 @@ class TestOptLat(unittest.TestCase):
         pmg3 = c3.to_pymatgen()
         self.assertTrue(sm.StructureMatcher().fit(pmg1, pmg3))
 
+    def test_molecular(self):
+        c1 = pyxtal(molecular=True)
+        c1.from_seed(seed=cif_path+"aspirin-c.cif", molecule="aspirin")
+        pmg1 = c1.to_pymatgen()
+
+        c2 = c1.copy()
+        c2.optimize_lattice(1)
+        pmg2 = c2.to_pymatgen()
+        self.assertTrue(sm.StructureMatcher().fit(pmg1, pmg2))
+
+    def test_molecular_diag(self):
+        sgs, diags = [5, 7, 8, 12, 13, 14], [True, False]
+        for diag in diags:
+            for i in range(40):
+                sg = choice(sgs)
+                num = len(Group(sg)[0])
+                c1 = pyxtal(molecular=True)
+                c1.from_random(3, sg, ["aspirin"], [num], diag=diag) 
+                pmg1 = c1.to_pymatgen()
+                c2 = c1.copy()
+                c2.optimize_lattice(1)
+                pmg2 = c2.to_pymatgen()
+                self.assertTrue(sm.StructureMatcher().fit(pmg1, pmg2))
+
+    def test_molecular_nodiag(self):
+        sgs, diag = [5, 7, 8, 12, 13, 14], False
+        for i in range(40):
+            sg = choice(sgs)
+            num = len(Group(sg)[0])
+            c1 = pyxtal(molecular=True)
+            c1.from_random(3, sg, ["aspirin"], [num], diag=diag) 
+            pmg1 = c1.to_pymatgen()
+            c2 = c1.copy()
+            c2.optimize_lattice(1)
+            pmg2 = c2.to_pymatgen()
+            self.assertTrue(sm.StructureMatcher().fit(pmg1, pmg2))
+
+
 class TestWP(unittest.TestCase):
     def test_wp(self):
         symbol = str(wp1.multiplicity) + wp1.letter
@@ -99,7 +139,7 @@ class TestMolecular(unittest.TestCase):
 
         # test rotation
         ax = struc.mol_sites[0].orientation.axis
-        struc.mol_sites[0].rotate(axis=[1, 0, 0], angle=90)
+        struc.mol_sites[0].rotate(ax_vector=[1, 0, 0], angle=90)
         pmg_struc = struc.to_pymatgen()
         sga = SpacegroupAnalyzer(pmg_struc)
         pmg_struc.to("cif", "1.cif")
@@ -123,6 +163,7 @@ class TestMolecular(unittest.TestCase):
         # test reading structure from external
         struc = pyxtal(molecular=True)
         struc.from_seed(seed=cif_path+"aspirin.cif", molecule="aspirin")
+        self.assertTrue(struc.lattice.ltype == "monoclinic")
         pmg_struc = struc.to_pymatgen()
         sga = SpacegroupAnalyzer(pmg_struc)
         self.assertTrue(sga.get_space_group_symbol() == "P2_1/c")
@@ -130,11 +171,11 @@ class TestMolecular(unittest.TestCase):
         pmg_s2 = C.to_pymatgen()
         self.assertTrue(sm.StructureMatcher().fit(pmg_struc, pmg_s2))
 
-    def test_ice(self):
-        struc = pyxtal(molecular=True)
-        struc.from_seed(seed=cif_path+"ice.cif", molecule='H2O')
-        N = struc.check_short_distances_by_dict({"H-H": 1.0, "O-O": 2.0})
-        self.assertTrue(N==0)
+    #def test_ice(self):
+    #    struc = pyxtal(molecular=True)
+    #    struc.from_seed(seed=cif_path+"ice.cif", molecule='H2O')
+    #    N = struc.check_short_distances_by_dict({"H-H": 1.0, "O-O": 2.0})
+    #    self.assertTrue(N==0)
 
     def test_big_molecule(self):
         # print("test_big_molecule")
@@ -332,9 +373,8 @@ class TestSubgroup(unittest.TestCase):
         C3s = C1.subgroup(permutations={"C":"Si"}, H=216)
 
     def test_from_seed(self):
-        from pymatgen import Lattice, Structure
         coords = [[0, 0, 0], [0.75,0.5,0.75]]
-        lattice = Lattice.from_parameters(a=3.84, b=3.84, c=3.84, alpha=120,
+        lattice = pmg_Lattice.from_parameters(a=3.84, b=3.84, c=3.84, alpha=120,
                                   beta=90, gamma=60)
         struct = Structure(lattice, ["Si", "C"], coords)
         s1 = pyxtal()
@@ -344,13 +384,13 @@ class TestSubgroup(unittest.TestCase):
         pmg_s2 = s2.to_pymatgen()
         self.assertTrue(sm.StructureMatcher().fit(pmg_s1, pmg_s2))
     
-        pmg_s1 = Structure.from_file(cif_path + "B28.vasp")
-        struc = pyxtal()
-        struc.from_seed(seed=cif_path + "B28.vasp")
-        pmg_s2 = struc.to_pymatgen()
-        self.assertTrue(sm.StructureMatcher().fit(pmg_s1, pmg_s2))
-        permutation = {"B":"C"}
-        struc.subgroup_once(0.01, None, permutation, max_cell=2) 
+        #pmg_s1 = Structure.from_file(cif_path + "B28.vasp")
+        #struc = pyxtal()
+        #struc.from_seed(seed=cif_path + "B28.vasp")
+        #pmg_s2 = struc.to_pymatgen()
+        #self.assertTrue(sm.StructureMatcher().fit(pmg_s1, pmg_s2))
+        #permutation = {"B":"C"}
+        #struc.subgroup_once(0.01, None, permutation, max_cell=2) 
 
     def test_molecules(self):
         for name in ["HAHCOI", "WEXBOS", "MERQIM", "LAGNAL", "YICMOP", "LUFHAW", "JAPWIH"]:
@@ -468,14 +508,22 @@ class Test_operations(unittest.TestCase):
         wp1, trans = wp.swap_axis([1,2,0])
         wp1, trans = wp.swap_axis([2,1,0])
 
-    def test_swap_xtal(self):
-        s = pyxtal()
-        s.from_seed(cif_path+"BTO-Amm2.cif")
-        strucs = s.get_alternatives()
-        self.assertTrue(len(strucs)==1)
-        #s.from_seed(cif_path+"BTO-Amm2.cif")
-        #strucs = s.get_alternatives()
-        #self.assertTrue(len(strucs)==5)
+    def test_alternative(self):
+        for name in ["BTO-Amm2", "lt_quartz", "GeF2", "lt_cristobalite", "PVO"]:
+            s = pyxtal()
+            s.from_seed(cif_path+name+'.cif')
+            pmg_s1 = s.to_pymatgen()
+            strucs = s.get_alternatives()
+            for struc in strucs:
+                pmg_s2 = struc.to_pymatgen()
+                #if not sm.StructureMatcher().fit(pmg_s1, pmg_s2):
+                #    print(struc)
+                #    print(s)
+                self.assertTrue(sm.StructureMatcher().fit(pmg_s1, pmg_s2))
+
+    def test_wyc_sets(self):
+        for i in range(1,229):
+            res = Group(i).get_alternatives()['No.']
 
 # class TestIO(unittest.TestCase):
 
